﻿using IRaCIS.Core.Domain.Share;
using IRaCIS.Core.Domain.Share.Reading;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IRaCIS.Core.Application.Service.Reading.Dto
{
    public class GetReadModuleDto:PageInput
    {
        public Guid? TrialId { get; set; }

        public Guid? SubjectId { get; set; }
    }



    public class ReadModuleAddDto
    {
        public Guid TrialId { get; set; }

        /// <summary>
        /// 截止日期
        /// </summary>
        public DateTime? ExpirationDate { get; set; }

        /// <summary>
        /// 截止访视
        /// </summary>
        public decimal? ExpirationVisit { get; set; }

        /// <summary>
        /// 访视计划ID
        /// </summary>
        public Guid? VisitStageId { get; set; }

        /// <summary>
        /// 受试者ID
        /// </summary>
        public Guid SubjectId { get; set; }


        /// <summary>
        /// 名称
        /// </summary>
        public string SubjectVisitName { get; set; }
    }


    public class ReadModuleAddOrEdit: ReadModule
    {
        public new Guid? Id { get; set; }

    }

    public class GetReadModuleSingleIndto
    {
        public  Guid Id { get; set; }

        public Guid SubjectId { get; set; }

       
    }


    public class GetReadModuleSingleOutdto
    {
        public Guid Id { get; set; }

      

        /// <summary>
        /// 模块类型
        /// </summary>
        public ModuleTypeEnum ModuleType { get; set; }

        /// <summary>
        /// 访视ID
        /// </summary>
        public Guid? SubjectVisitId { get; set; }


        /// <summary>
        /// 访视名称
        /// </summary>
        public string SubjectVisitName { get; set; }

        /// <summary>
        /// 干系人
        /// </summary>
        public string Stakeholders { get; set; }
    }

    public class GetReadModuleOutDto 
    {

        /// <summary>
        /// Id(阅片期Id 或者 访视ID)
        /// </summary>
        public Guid? Id { get; set; }

        /// <summary>
        /// 名称
        /// </summary>
        public string Name { get; set; }

        /// <summary>
		/// 受试者ID
		/// </summary>
		public Guid? SubjectId { get; set; }

        /// <summary>
        /// 模块类型
        /// </summary>
        public ModuleTypeEnum ModuleType { get; set; }

        /// <summary>
        /// 是否加急
        /// </summary>
        public bool? IsUrgent { get; set; }

        /// <summary>
        /// 访视ID
        /// </summary>
        public Guid? SubjectVisitId { get; set; }


        /// <summary>
        /// 访视名称
        /// </summary>
        public string SubjectVisitName { get; set; }


        /// <summary>
        /// 截止访视
        /// </summary>
        public Guid? CutOffVisitId { get; set; }


        /// <summary>
        /// 截止访视名称
        /// </summary>
        public string CutOffVisitName { get; set; }

        /// <summary>
        /// 对应阅片期
        /// </summary>
        public Guid? ReadModuleId { get; set; }

        /// <summary>
        /// 对应Name
        /// </summary>
        public string ReadModuleName { get; set; }

        /// <summary>
        /// 状态
        /// </summary>
        public ReadModuleEnum? Status { get; set; }

        /// <summary>
        /// 创建时间
        /// </summary>
        public DateTime CreateTime { get; set; }

        /// <summary>
        /// 受试者Code
        /// </summary>
        public string SubjectCode { get; set; }

        /// <summary>
        /// 中心Id
        /// </summary>
        public string SiteCode { get; set; }

        /// <summary>
        /// 是否为末次访视
        /// </summary>
        public bool IsFinalVisit { get; set; } = false;


        /// <summary>
        /// 上一访视
        /// </summary>
        public Guid? OutPlanPreviousVisitId { get; set; }

        /// <summary>
        /// 上一访视名称
        /// </summary>
        public string OutPlanPreviousVisitName { get; set; }

        /// <summary>
        /// 中心ID
        /// </summary>
        public Guid? SiteId { get; set; }

        /// <summary>
        /// 是否为访视
        /// </summary>
        public bool IsVisit { get; set; }

        /// <summary>
        /// 是否为PD进展
        /// </summary>
        public PDStateEnum PDState { get; set; } = PDStateEnum.None;

        /// <summary>
        /// 入组确认评估
        /// </summary>
        public bool IsEnrollmentConfirm { get; set; } = false;


    }

    
    public class GetReadModuleResultDto
    {
        /// <summary>
        /// 当前页索引
        /// </summary>
        public int PageIndex { get; set; }

        /// <summary>
        /// 每页的记录条数
        /// </summary>
        public int PageSize { get; set; }

        /// <summary>
        /// 数据总数
        /// </summary>
        public long TotalCount { get; set; } = 0;

        /// <summary>
        /// 最大长度
        /// </summary>
        public int MaxLength { get; set; }

        /// <summary>
        /// 数据
        /// </summary>
        public List<GetReadModuleDtoOut> CurrentPageData { get; set; }
    }
        
        

    public class GetReadModuleDtoOut
    {
        public string SubjectCode { get; set; }

        public string SiteCode { get; set; }

        public Guid? SiteId { get; set; }

        public Guid? SubjectId { get; set; }

        public List<GetReadModuleOutDto> Data { get; set; }
    }
}
